@@ -1,31 +1,33 @@
 /*
-  Guifications - The end all, be all, toaster popup plugin
-  Copyright (C) 2003-2004 Gary Kramlich
-
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License
-  as published by the Free Software Foundation; either version 2
-  of the License, or (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * Guifications - The end all, be all, toaster popup plugin
+ * Copyright (C) 2003-2004 Gary Kramlich
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 #ifndef _WIN32
+# include <gdk/gdkx.h>
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <X11/Xatom.h>
-# include <gdk/gdkx.h>
 #else
+# ifndef WINVER
+#  define WINVER 0x0500 /* This is Windows 2000 */
+# endif /* WINVER */
 # include <windows.h>
-# ifndef SPI_GETSCREENSAVERRUNNING
-#  define SPI_GETSCREENSAVERRUNNING 114
-# endif
+typedef HMONITOR WINAPI GF_MonitorFromRect(LPCRECT,DWORD);
+typedef BOOL WINAPI GF_GetMonitorInfo(HMONITOR,LPMONITORINFO);
 #endif /* _WIN32 */
 
 #include <glib.h>
@@ -63,7 +65,9 @@ struct _GfDisplay {
  * Globals
  ******************************************************************************/
 static GList *displays = NULL;
+#if GTK_CHECK_VERSION(2,2,0)
 static gint disp_screen = 0, disp_monitor = 0;
+#endif
 
 /*******************************************************************************
  * Callbacks
@@ -108,58 +112,239 @@ gf_display_button_press_cb(GtkWidget *w, GdkEventButton *e, gpointer data) {
 }
 
 /*******************************************************************************
- * Uses _NET_WORKAREA to get the geometry of a screen
+ * Uses _NET_WORKAREA or SPI_GETWORKAREA to get the geometry of a screen
  ******************************************************************************/
+#ifdef _WIN32
+static gboolean
+win32_get_workarea_fallback(GdkRectangle *rect) {
+	RECT rcWork;
+	if (SystemParametersInfo(SPI_GETWORKAREA, 0, &rcWork, FALSE)) {
+		rect->x = rcWork.left;
+		rect->y = rcWork.top;
+		rect->width = rcWork.right - rcWork.left;
+		rect->height = rcWork.bottom - rcWork.top;
+
+		return TRUE;
+	}
+	return FALSE;
+}
+
+/**
+ * This will only work on Win98+ and Win2K+
+ */
+static gboolean
+win32_adjust_workarea_multi_monitor(GdkRectangle *rect) {
+	GF_GetMonitorInfo *the_GetMonitorInfo;
+	GF_MonitorFromRect *the_MonitorFromRect;
+	HMODULE hmod;
+	HMONITOR monitor;
+	MONITORINFO info;
+	RECT monitorRect;
+	gint virtual_screen_x, virtual_screen_y;
+
+	monitorRect.left = rect->x;
+	monitorRect.right = rect->x + rect->width;
+	monitorRect.top = rect->y;
+	monitorRect.bottom = rect->y + rect->height;
+
+	/* Convert the coordinates so that 0, 0 is the top left of the Primary
+	 * Monitor, not the top left of the virtual screen
+	 */
+	virtual_screen_x = GetSystemMetrics(SM_XVIRTUALSCREEN);
+	virtual_screen_y = GetSystemMetrics(SM_YVIRTUALSCREEN);
+
+	if (virtual_screen_x != 0) {
+		monitorRect.left += virtual_screen_x;
+		monitorRect.right += virtual_screen_x;
+	}
+	if (virtual_screen_y != 0) {
+		monitorRect.top += virtual_screen_y;
+		monitorRect.bottom += virtual_screen_y;
+	}
+
+	if (!(hmod = GetModuleHandle("user32"))) {
+		return FALSE;
+	}
+
+	if (!(the_MonitorFromRect = (GF_MonitorFromRect*)
+		GetProcAddress(hmod, "MonitorFromRect"))) {
+		return FALSE;
+	}
+
+	if (!(the_GetMonitorInfo = (GF_GetMonitorInfo*)
+		GetProcAddress(hmod, "GetMonitorInfoA"))) {
+		return FALSE;
+	}
+
+	monitor = the_MonitorFromRect(&monitorRect, MONITOR_DEFAULTTONEAREST);
+
+	info.cbSize = sizeof(info);
+	if (!the_GetMonitorInfo(monitor, &info)) {
+		return FALSE;
+	}
+
+	rect->x = info.rcWork.left;
+	rect->y = info.rcWork.top;
+	rect->width = info.rcWork.right - info.rcWork.left;
+	rect->height = info.rcWork.bottom - info.rcWork.top;
+
+	/** Convert the coordinates so that 0, 0 is the top left of the virtual screen,
+	 * not the top left of the primary monitor */
+	if (virtual_screen_x != 0) {
+		rect->x += -virtual_screen_x;
+	}
+	if (virtual_screen_y != 0) {
+		rect->y += -virtual_screen_y;
+	}
+
+	return TRUE;
+}
+
+static void
+win32_adjust_workarea(GdkRectangle *rect) {
+	if (!win32_adjust_workarea_multi_monitor(rect)) {
+		if (!win32_get_workarea_fallback(rect)) {
+			/* I don't think this will ever happen */
+			rect->x = 0;
+			rect->y = 0;
+			rect->height = GetSystemMetrics(SM_CXSCREEN);
+			rect->width = GetSystemMetrics(SM_CYSCREEN);
+		}
+	}
+}
+#endif /** _WIN32 */
+
 gboolean
 gf_display_get_workarea(GdkRectangle *rect) {
 #ifndef _WIN32
-	Atom workarea, type_return;
-	Window win;
-	guchar *workarea_return;
-	gulong leftovers, num;
-	glong max_len = 4 * 32;
-	guint32 *workareas;
-	gint result, format_return;
+	Atom xa_desktops, xa_current, xa_workarea, xa_type;
+	Display *x_display;
+	Window x_root;
+	guint32 *workareas, desktops = 0, current = 0;
+	gulong len, fill;
+	guchar *data;
+	gint format;
 
-	g_return_val_if_fail(rect, FALSE);
+# if !GTK_CHECK_VERSION(2,2,0)
 
-	workarea = XInternAtom(GDK_DISPLAY(), "_NET_WORKAREA", True);
-	if(workarea == None)
+	x_display = GDK_DISPLAY();
+	x_root = XDefaultRootWindow(x_display);
+
+# else /* GTK 2.2.0 and up */
+
+	GdkDisplay *g_display;
+	GdkScreen *g_screen;
+	Screen *x_screen;
+
+	/* get the gdk display */
+	g_display = gdk_display_get_default();
+	if(!g_display)
 		return FALSE;
 
-	win = XRootWindow(GDK_DISPLAY(), disp_screen);
-	result = XGetWindowProperty(GDK_DISPLAY(), win, workarea,
-								0, max_len, False, AnyPropertyType,
-								&type_return, &format_return, &num,
-								&leftovers, &workarea_return);
-
-	/* talk about error checking... */
-	if(result != Success)
+	/* get the x display from the gdk display */
+	x_display = gdk_x11_display_get_xdisplay(g_display);
+	if(!x_display)
 		return FALSE;
 
-	if(type_return == None || format_return == 0)
+	/* get the screen according to the prefs */
+	g_screen = gdk_display_get_screen(g_display, disp_screen);
+	if(!g_screen)
 		return FALSE;
 
-	if(leftovers)
+	/* get the x screen from the gdk screen */
+	x_screen = gdk_x11_screen_get_xscreen(g_screen);
+	if(!x_screen)
 		return FALSE;
 
-	if(num % 4)
+	/* get the root window from the screen */
+	x_root = XRootWindowOfScreen(x_screen);
+
+# endif /* !GTK_CHECK_VERSION(2,2,0) */
+
+	/* find the _NET_NUMBER_OF_DESKTOPS atom */
+	xa_desktops = XInternAtom(x_display, "_NET_NUMBER_OF_DESKTOPS", True);
+	if(xa_desktops == None)
 		return FALSE;
 
-	/* everything passed, woohoo! */
-	workareas = (guint32 *)workarea_return;
+	/* get the number of desktops */
+	if(XGetWindowProperty(x_display, x_root, xa_desktops, 0, 1, False,
+						  XA_CARDINAL, &xa_type, &format, &len, &fill,
+						  &data) != Success)
+	{
+		return FALSE;
+	}
 
-	rect->x = workareas[disp_screen * 4];
-	rect->y = workareas[disp_screen * 4 + 1];
-	rect->width = workareas[disp_screen * 4 + 2];
-	rect->height = workareas[disp_screen * 4 + 3];
+	if(!data)
+		return FALSE;
 
-	XFree(workarea_return);
+	desktops = *(guint32 *)data;
+	XFree(data);
+
+	/* find the _NET_CURRENT_DESKTOP atom */
+	xa_current = XInternAtom(x_display, "_NET_CURRENT_DESKTOP", True);
+	if(xa_current == None)
+		return FALSE;
+
+	/* get the current desktop */
+	if(XGetWindowProperty(x_display, x_root, xa_current, 0, 1, False,
+						  XA_CARDINAL, &xa_type, &format, &len, &fill,
+						  &data) != Success)
+	{
+		return FALSE;
+	}
+
+	if(!data)
+		return FALSE;
+
+	current = *(guint32 *)data;
+	XFree(data);
+
+	/* find the _NET_WORKAREA atom */
+	xa_workarea = XInternAtom(x_display, "_NET_WORKAREA", True);
+	if(xa_workarea == None)
+		return FALSE;
+
+	if(XGetWindowProperty(x_display, x_root, xa_workarea, 0, (glong)(4 * 32),
+						  False, AnyPropertyType, &xa_type, &format, &len,
+						  &fill, &data) != Success)
+	{
+		return FALSE;
+	}
+
+	/* make sure the type and format are good */
+	if(xa_type == None || format == 0)
+		return FALSE;
+
+	/* make sure we don't have any leftovers */
+	if(fill)
+		return FALSE;
+
+	/* make sure len divides evenly by 4 */
+	if(len % 4)
+		return FALSE;
+
+	/* it's good, lets use it */
+	workareas = (guint32 *)data;
+
+	rect->x = workareas[current * 4];
+	rect->y = workareas[current * 4 + 1];
+	rect->width = workareas[current * 4 + 2];
+	rect->height = workareas[current * 4 + 3];
+
+	/* clean up our memory */
+	XFree(data);
+#else
+	GdkDisplay *display;
+	GdkScreen *screen;
+
+	display = gdk_display_get_default();
+	screen = gdk_display_get_screen(display, disp_screen);
+	gdk_screen_get_monitor_geometry(screen, disp_monitor, rect);
+
+	win32_adjust_workarea(rect);
+#endif /* _WIN32 */
 
 	return TRUE;
-#else
-	return FALSE;
-#endif /* _WIN32 */
 }
 
 /*******************************************************************************
@@ -197,7 +382,7 @@ gf_display_move(GfDisplay *display) {
 /*******************************************************************************
  * Gtk 2.2.0 and up stuff
  *
- * Allows users to specific which screen to display the notifications on and
+ * Allows users to specify which screen to display the notifications on and
  * in the case of xinerama, to position it correctly
  ******************************************************************************/
 
@@ -205,22 +390,25 @@ static void
 gf_display_get_geometry(gint *x, gint *y, gint *width, gint *height) {
 	GdkDisplay *display;
 	GdkScreen *screen;
-	GdkRectangle geometry;
+	GdkRectangle geo, m_geo, w_geo;
 
-#ifndef _WIN32
-	if(!gf_display_get_workarea(&geometry)) {
-#endif /* _WIN32 */
-		display = gdk_display_get_default();
-		screen = gdk_display_get_screen(display, disp_screen);
-		gdk_screen_get_monitor_geometry(screen, disp_monitor, &geometry);
-#ifndef _WIN32
+	display = gdk_display_get_default();
+	screen = gdk_display_get_screen(display, disp_screen);
+	gdk_screen_get_monitor_geometry(screen, disp_monitor, &m_geo);
+
+	if(gf_display_get_workarea(&w_geo)) {
+		gdk_rectangle_intersect(&w_geo, &m_geo, &geo);
+	} else {
+		geo.x = m_geo.x;
+		geo.y = m_geo.y;
+		geo.width = m_geo.width;
+		geo.height = m_geo.height;
 	}
-#endif /* _WIN32 */
 
-	*x = geometry.x;
-	*y = geometry.y;
-	*width = geometry.width;
-	*height = geometry.height;
+	*x = geo.x;
+	*y = geo.y;
+	*width = geo.width;
+	*height = geo.height;
 }
 
 static void
@@ -263,11 +451,25 @@ gf_display_move(GfDisplay *display) {
 	gtk_window_move(GTK_WINDOW(display->window), display->x, display->y);
 }
 
+gint
+gf_display_get_default_screen() {
+	GdkScreen *screen;
+
+	screen = gdk_screen_get_default();
+
+	return gdk_screen_get_number(screen);
+}
+
 gint
 gf_display_get_screen_count() {
 	return gdk_display_get_n_screens(gdk_display_get_default()) - 1;
 }
 
+gint
+gf_display_get_default_monitor() {
+	return 0;
+}
+
 gint
 gf_display_get_monitor_count() {
 	GdkDisplay *display;
@@ -422,41 +624,6 @@ gf_display_destroy_cb(gpointer data) {
 	return FALSE;
 }
 
-static void
-gf_display_zoom(GfDisplay *display) {
-	GdkPixbuf *scaled = NULL;
-	gfloat multiplier, height, width;
-
-	g_return_if_fail(display);
-
-	switch(gaim_prefs_get_int(GF_PREF_APPEARANCE_ZOOM)) {
-		case GF_DISPLAY_ZOOM_200:	multiplier = 2.0;	break;
-		case GF_DISPLAY_ZOOM_175:	multiplier = 1.75;	break;
-		case GF_DISPLAY_ZOOM_150:	multiplier = 1.5;	break;
-		case GF_DISPLAY_ZOOM_125:	multiplier = 1.25;	break;
-		case GF_DISPLAY_ZOOM_75:	multiplier = 0.75;	break;
-		case GF_DISPLAY_ZOOM_50:	multiplier = 0.5;	break;
-		case GF_DISPLAY_ZOOM_25:	multiplier = 0.25;	break;
-		case GF_DISPLAY_ZOOM_100:
-		default:
-			return;
-			break;
-	}
-
-	gaim_debug_info("Guifications", "Zoom isn't 100 percent, scaling now\n");
-
-	height = gdk_pixbuf_get_height(display->pixbuf) * multiplier;
-	width = gdk_pixbuf_get_width(display->pixbuf) * multiplier;
-
-	scaled = gdk_pixbuf_scale_simple(display->pixbuf, width, height,
-									 GDK_INTERP_BILINEAR);
-
-	if(scaled) {
-		g_object_unref(G_OBJECT(display->pixbuf));
-		display->pixbuf = scaled;
-	}
-}
-
 static void
 gf_display_condense_contacts(GfEventInfo *info) {
 	GaimBuddy *buddy1, *buddy2;
@@ -505,8 +672,7 @@ gf_display_condense(GfEventInfo *info) {
 	GList *l, *ll;
 	const gchar *target;
 
-	if(gaim_prefs_get_bool(GF_PREF_APPEARANCE_CONTACTS))
-		gf_display_condense_contacts(info);
+	gf_display_condense_contacts(info);
 
 	buddy = gf_event_info_get_buddy(info);
 	conv = gf_event_info_get_conversation(info);
@@ -563,7 +729,7 @@ gf_display_throttle() {
 	gf_displays_position();
 }
 
-static gboolean
+gboolean
 gf_display_screen_saver_is_running() {
 	gboolean ret = FALSE;
 # ifndef _WIN32
@@ -594,36 +760,41 @@ gf_display_screen_saver_is_running() {
 		XFree(data);
 	}
 #else
-	if (!SystemParametersInfo(SPI_GETSCREENSAVERRUNNING,
-			(unsigned int) NULL,
-			&ret,
-			FALSE))
+	if(!SystemParametersInfo(SPI_GETSCREENSAVERRUNNING, (unsigned int) NULL,
+							 &ret, FALSE))
+	{
 		ret = FALSE;
+	}
 #endif /* _WIN32 */
 	return ret;
 }
 
 void
-gf_display_show_event(GfEventInfo *info) {
+gf_display_show_event(GfEventInfo *info, GfNotification *notification) {
 	GfDisplay *display = NULL;
-	GfEvent *event;
-	GfNotification *notification = NULL;
-	const gchar *n_type;
 	gint display_time;
 	guint timeout_id = 0;
 
 	g_return_if_fail(info);
 
-	/* Here we kill the notification if the screen saver is running and the user doesn't
-	 * want them showing while it's running.
-	 */
-	if(!gaim_prefs_get_bool(GF_PREF_BEHAVIOR_SHOW_DURING_SCREEN_SAVER) &&
-	   gf_display_screen_saver_is_running())
-	{
+	/* Here we kill the notification if the screen saver is running */
+	if(gf_display_screen_saver_is_running()) {
 		gf_event_info_destroy(info);
 		return;
 	}
 
+	/* we should never make it here with out a notification, but just in 
+	 * case we do....
+	 */
+	if(!notification) {
+		const gchar *event_name = gf_event_get_name(gf_event_info_get_event(info));
+
+		gaim_debug_info("Guifications",
+						"could not find a notification for the event \"%s\"\n",
+						event_name ? event_name : "");
+		return;
+	}
+
 	/* condense contacts and same buddy notifications */
 	gf_display_condense(info);
 
@@ -634,20 +805,6 @@ gf_display_show_event(GfEventInfo *info) {
 	display = gf_display_new();
 	display->info = info;
 
-	/* find a notification for the event */
-	event = gf_event_info_get_event(info);
-	n_type = gf_event_get_notification_type(event);
-	notification = gf_notification_find_for_event(n_type);
-
-	/* if we don't find an event, destroy the display and return */
-	if(!notification) {
-		gaim_debug_info("Guifications",
-						"could not find a notification for the event \"%s\"\n",
-						n_type);
-		gf_display_destroy(display);
-		return;
-	}
-
 	/* Render the pixbuf, if we get NULL destroy the display and return */
 	display->pixbuf = gf_notification_render(notification, display->info);
 	if(!display->pixbuf) {
@@ -655,21 +812,22 @@ gf_display_show_event(GfEventInfo *info) {
 		GfThemeInfo *info = gf_theme_get_theme_info(theme);
 
 		gaim_debug_info("Guifications", "render '%s' failed for theme '%s'\n",
-						n_type, gf_theme_info_get_name(info));
+						gf_notification_get_type(notification), 
+						gf_theme_info_get_name(info));
 		gf_display_destroy(display);
 		return;
 	}
 
 	/* grab info about the pixbuf and scale it */
-	gf_display_zoom(display);
 	display->has_alpha = gdk_pixbuf_get_has_alpha(display->pixbuf);
 	display->height = gdk_pixbuf_get_height(display->pixbuf);
 	display->width = gdk_pixbuf_get_width(display->pixbuf);
 
 	/* if we've made it this far, we can create the window safely */
 	display->window = gtk_window_new(GTK_WINDOW_POPUP);
+	gtk_window_set_role(GTK_WINDOW(display->window), "guification");
 
-	/* now we create all our mouse signals that should play nice with other apps.. */
+    /* now we create all our mouse signals that should play nice with other apps.. */
 	g_signal_connect(G_OBJECT(display->window), "button-press-event",
 					 G_CALLBACK(gf_display_button_press_cb), (gpointer)display);
 	g_signal_connect(G_OBJECT(display->window), "button-release-event",
@@ -711,7 +869,7 @@ static void
 gf_display_position_changed_cb(const gchar *name, GaimPrefType type,
 							   gpointer val, gpointer data)
 {
-	gf_displays_position();
+    gf_displays_position();
 }
 
 /*******************************************************************************
