@@ -1,21 +1,21 @@
 /*
-  Guifications - The end all, be all, toaster popup plugin
-  Copyright (C) 2003-2004 Gary Kramlich
-
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License
-  as published by the Free Software Foundation; either version 2
-  of the License, or (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * Guifications - The end all, be all, toaster popup plugin
+ * Copyright (C) 2003-2004 Gary Kramlich
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 #include <glib.h>
 #include <string.h>
 
@@ -28,9 +28,10 @@
 #include <prefs.h>
 #include <util.h>
 
+#include "gf_blist.h"
 #include "gf_display.h"
 #include "gf_event.h"
-#include "gf_intl.h"
+#include "gf_internal.h"
 #include "gf_notification.h"
 #include "gf_preferences.h"
 #include "gf_release.h"
@@ -227,23 +228,6 @@ gf_events_get_nth_notification(gint nth) {
 	return event->n_type;
 }
 
-/*******************************************************************************
- * Helpers
- ******************************************************************************/
-static gboolean
-gf_event_show_while_away(GaimAccount *account) {
-	gboolean pref = gaim_prefs_get_bool(GF_PREF_BEHAVIOR_SHOW_WHILE_AWAY);
-
-	if(pref) {
-		return TRUE;
-	} else {
-		if(account->gc->away_state)
-			return FALSE;
-		else
-			return TRUE;
-	}
-}
-
 /*******************************************************************************
  * Signon flood be gone!
  ******************************************************************************/
@@ -282,8 +266,7 @@ gf_event_connection_throttle(GaimConnection *gc, gpointer data) {
 		return;
 
 	accounts = g_list_append(accounts, account);
-	g_timeout_add(gaim_prefs_get_int(GF_PREF_ADVANCED_SIGN_ON_DELAY),
-				  gf_event_connection_throttle_cb, (gpointer)account);
+	g_timeout_add(10000, gf_event_connection_throttle_cb, (gpointer)account);
 }
 
 /*******************************************************************************
@@ -313,89 +296,138 @@ gf_event_conversation_throttle(GaimConversation *conv, gpointer data) {
 	g_timeout_add(5000, gf_event_conversation_throttle_cb, (gpointer)conv);
 }
 
+/*******************************************************************************
+ * Helpers
+ ******************************************************************************/
+/* first pass to see if a notification should be shown.. */
+static gboolean
+gf_event_should_show(const gchar *notification, GaimAccount *account) {
+	if(gf_display_screen_saver_is_running())
+		return FALSE;
+
+	if(g_list_find(accounts, account))
+		return FALSE;
+
+	if(!gf_event_show_notification(notification))
+		return FALSE;
+
+	return TRUE;
+}
+
+/* second pass to see if a conversation notification should be shown */
+static gboolean
+gf_event_conversation_show(GaimConversation *conv, const gchar *sender) {
+	if(!conv)
+		return FALSE;
+
+	if(gaim_conversation_has_focus(conv))
+		return FALSE;
+
+	if(gaim_conversation_get_type(conv) == GAIM_CONV_CHAT) {
+		if(g_list_find(chats, conv))
+			return FALSE;
+
+		if(sender) {
+			const gchar *nick;
+
+			nick = gaim_conv_chat_get_nick(GAIM_CONV_CHAT(conv));
+			if(!strcmp(nick, sender))
+				return FALSE;
+        }
+	}
+
+	return TRUE;
+}
+
+static void
+gf_event_common(gchar *n_type, GaimAccount *account, GaimBuddy *buddy,
+				GaimConversation *conv, const gchar *target,
+				const gchar *message, GaimConvChatBuddyFlags flags,
+				const GHashTable *components, const gchar *extra)
+{
+	GfNotification *notification = NULL;
+	GfEventInfo *info = NULL;
+
+	g_return_if_fail(n_type);
+	g_return_if_fail(account);
+
+	if(!gf_event_should_show(n_type, account))
+		return;
+
+	if(conv && target) {
+		if(!gf_event_conversation_show(conv, target))
+			return;
+	}
+
+	if(buddy) {
+		notification = gf_blist_get_notification_for_buddy(buddy, n_type);
+	} else {
+		notification = gf_notification_find_for_event(n_type);
+	}
+
+	if(!notification)
+		return;
+
+	info = gf_event_info_new(n_type);
+
+	gf_event_info_set_account(info, account);
+
+	if(buddy)
+		gf_event_info_set_buddy(info, buddy);
+
+	if(conv)
+		gf_event_info_set_conversation(info, conv);
+
+	if(target)
+		gf_event_info_set_target(info, target);
+
+	if(message)
+		gf_event_info_set_message(info, message);
+
+	gf_event_info_set_conv_chat_buddy_flags(info, flags);
+
+	if(components)
+		gf_event_info_set_components(info, components);
+
+	if(extra)
+		gf_event_info_set_extra(info, extra);
+
+	gf_display_show_event(info, notification);
+}
+
 /*******************************************************************************
  * Buddy callbacks
  ******************************************************************************/
 static void
 gf_event_buddy(GaimBuddy *buddy, gpointer data) {
-	GfEventInfo *info;
 	gchar *notification = (gchar *)data;
 
-	g_return_if_fail(data);
-
-	if(g_list_find(accounts, buddy->account))
+	if(!gf_event_should_show(notification, buddy->account))
 		return;
 
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(buddy->account))
-		return;
-
-	info = gf_event_info_new(notification);
-	gf_event_info_set_account(info, buddy->account);
-	gf_event_info_set_buddy(info, buddy);
-	gf_event_info_set_target(info, buddy->name);
-
-	gf_display_show_event(info);
+	gf_event_common(notification, buddy->account, buddy, NULL, buddy->name,
+					NULL, GAIM_CBFLAGS_NONE, NULL, NULL);
 }
 
 /*******************************************************************************
  * Conversation callbacks
  ******************************************************************************/
-static gboolean
-gf_event_conversation_show_notification(GaimConversation *conv) {
-	if(gaim_prefs_get_bool(GF_PREF_ADVANCED_HIDE_CONV_FOCUSED)) {
-		if(gaim_conversation_has_focus(conv))
-			return FALSE;
-		else
-			return TRUE;
-	} else {
-		return TRUE;
-	}
-}
-
 static void
 gf_event_im_message(GaimAccount *account, const gchar *sender,
 					const gchar *message, int flags, gpointer data)
 {
-	GfEventInfo *info;
 	GaimConversation *conv = NULL;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
 
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	info = gf_event_info_new(notification);
+	conv = gaim_find_conversation_with_account(sender, account);
+	buddy = gaim_find_buddy(account, sender);
 	plain_message = gaim_markup_strip_html(message);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_target(info, sender);
-
-	if((buddy = gaim_find_buddy(account, sender)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, conv, sender, plain_message,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 	g_free(plain_message);
-
-	conv = gaim_find_conversation_with_account(sender, account);
-	if(conv) {
-		gf_event_info_set_conversation(info, conv);
-		if(!gf_event_conversation_show_notification(conv)) {
-			gf_event_info_destroy(info);
-			return;
-		}
-	}
-
-	gf_display_show_event(info);
 }
 
 static void
@@ -403,218 +435,86 @@ gf_event_chat_message(GaimAccount *account, const gchar *sender,
 					  const gchar *message, GaimConversation *conv,
 					  gpointer data)
 {
-	GfEventInfo *info;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
-	const gchar *nick;
 
-	g_return_if_fail(data);
-
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	if(!conv && !gf_event_conversation_show_notification(conv))
-		return;
-
-	nick = gaim_conv_chat_get_nick(GAIM_CONV_CHAT(conv));
-	if (nick != NULL && !g_ascii_strcasecmp(sender, nick))
-		return;
-
-	info = gf_event_info_new(notification);
 	plain_message = gaim_markup_strip_html(message);
+	buddy = gaim_find_buddy(account, sender);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_conversation(info, conv);
-
-	if((buddy = gaim_find_buddy(account, sender)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_event_info_set_target(info, sender);
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, conv, sender, plain_message,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 	g_free(plain_message);
-
-	gf_display_show_event(info);
 }
 
 static void
 gf_event_chat_nick(GaimAccount *account, const gchar *sender,
-					  const gchar *message, GaimConversation *conv,
-					  gpointer data)
+				   const gchar *message, GaimConversation *conv,
+				   gpointer data)
 {
-	GfEventInfo *info;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
-	const gchar *nick;
-
-	g_return_if_fail(data);
-
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	if(!conv && !gf_event_conversation_show_notification(conv))
-		return;
+	const gchar *nick = NULL;
 
 	nick = gaim_conv_chat_get_nick(GAIM_CONV_CHAT(conv));
-	if (nick && !g_ascii_strcasecmp(sender, nick))
+	if (nick && !strcmp(sender, nick))
 		return;
 
-	if(g_strstr_len(message, strlen(message), nick) == NULL)
+	if(!g_strstr_len(message, strlen(message), nick))
 		return;
 
-	info = gf_event_info_new(notification);
 	plain_message = gaim_markup_strip_html(message);
+	buddy = gaim_find_buddy(account, sender);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_conversation(info, conv);
-
-	if((buddy = gaim_find_buddy(account, sender)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_event_info_set_target(info, sender);
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, conv, sender, plain_message,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 	g_free(plain_message);
-
-	gf_display_show_event(info);
 }
 
 static void
 gf_event_typing(GaimConversation *conv, gpointer data) {
-	GfEventInfo *info;
-	GaimAccount *account;
+	GaimAccount *account = NULL;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 
-	g_return_if_fail(data);
-
-	if(!conv)
-		return;
-
-	if(!gf_event_conversation_show_notification(conv))
-		return;
-
 	account = gaim_conversation_get_account(conv);
+	buddy = gaim_find_buddy(account, conv->name);
 
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	info = gf_event_info_new(notification);
-	gf_event_info_set_account(info, account);
-	if(conv)
-		gf_event_info_set_conversation(info, conv);
-	if((buddy = gaim_find_buddy(account, conv->name)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_display_show_event(info);
+	gf_event_common(notification, account, buddy, conv, conv->name, NULL,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 }
 
-static gboolean
+static void
 gf_event_chat_join(GaimConversation *conv, const gchar *name,
 				   GaimConvChatBuddyFlags flags, gpointer data) {
-	GfEventInfo *info;
-	GaimAccount *account;
+	GaimAccount *account = NULL;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 
-	g_return_val_if_fail(data, FALSE);
-
-	if(!conv)
-		return FALSE;
-
-	if(!gf_event_conversation_show_notification(conv))
-		return FALSE;
-
 	account = gaim_conversation_get_account(conv);
+	buddy = gaim_find_buddy(account, name);
 
-	if(g_list_find(accounts, account))
-		return FALSE;
-
-	if(g_list_find(chats, conv))
-		return FALSE;
-
-	if(!gf_event_show_notification(notification))
-		return FALSE;
-
-	if(!gf_event_show_while_away(account))
-		return FALSE;
-
-	info = gf_event_info_new(notification);
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_conversation(info, conv);
-	gf_event_info_set_target(info, name);
-	gf_event_info_set_conv_chat_buddy_flags(info, flags);
-
-	if((buddy = gaim_find_buddy(account, name)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_display_show_event(info);
-
-	return gaim_prefs_get_bool(GF_PREF_ADVANCED_HIDE_JOINLEAVE);
+	gf_event_common(notification, account, buddy, conv, name, NULL, flags,
+					NULL, NULL);
 }
 
-static gboolean
+static void
 gf_event_chat_part(GaimConversation *conv, const gchar *name,
 				   const gchar *reason, gpointer data)
 {
-	GfEventInfo *info;
-	GaimAccount *account;
-	GaimBuddy *buddy = NULL;
+	GaimAccount *account = NULL;
+    GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
 
-	g_return_val_if_fail(data, FALSE);
-
-	if(!conv)
-		return FALSE;
-
-	if(!gf_event_conversation_show_notification(conv))
-		return FALSE;
-
 	account = gaim_conversation_get_account(conv);
-
-	if(g_list_find(accounts, account))
-		return FALSE;
-
-	if(!gf_event_show_notification(notification))
-		return FALSE;
-
-	if(!gf_event_show_while_away(account))
-		return FALSE;
-
-	info = gf_event_info_new(notification);
 	plain_message = (reason) ? gaim_markup_strip_html(reason) : g_strdup("");
+	buddy = gaim_find_buddy(account, name);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_conversation(info, conv);
-	gf_event_info_set_target(info, name);
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, conv, name, plain_message,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 	g_free(plain_message);
-
-	if((buddy = gaim_find_buddy(account, name)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_display_show_event(info);
-
-	return gaim_prefs_get_bool(GF_PREF_ADVANCED_HIDE_JOINLEAVE);
 }
 
 static void
@@ -622,75 +522,36 @@ gf_event_chat_invite(GaimAccount *account, const gchar *inviter,
 					 const gchar *chat, const gchar *invite_message,
 					 const GHashTable *components, gpointer data)
 {
-	GfEventInfo *info;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
 
-	g_return_if_fail(data);
-
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	info = gf_event_info_new(notification);
 	plain_message = (invite_message) ? gaim_markup_strip_html(invite_message) :
 					g_strdup("");
+	buddy = gaim_find_buddy(account, inviter);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_target(info, inviter);
-	gf_event_info_set_extra(info, chat);
-	gf_event_info_set_components(info, components);
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, NULL, inviter, plain_message,
+					GAIM_CBFLAGS_NONE, components, chat);
 	g_free(plain_message);
-
-	if((buddy = gaim_find_buddy(account, inviter)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_display_show_event(info);
 }
 
 static void
 gf_event_topic_changed(GaimConversation *conv, const gchar *who,
 					   const gchar *topic, gpointer data)
 {
-	GfEventInfo *info;
-	GaimAccount *account;
+	GaimAccount *account = NULL;
 	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 	gchar *plain_message = NULL;
 
-	g_return_if_fail(data);
-
 	account = gaim_conversation_get_account(conv);
-
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	info = gf_event_info_new(notification);
 	plain_message = (topic) ? gaim_markup_strip_html(topic) : g_strdup("");
+	if(who)
+		buddy = gaim_find_buddy(account, who);
 
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_conversation(info, conv);
-	gf_event_info_set_target(info, who ? who : "");
-	gf_event_info_set_message(info, plain_message);
+	gf_event_common(notification, account, buddy, conv, who, plain_message,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 	g_free(plain_message);
-
-	if(who && (buddy = gaim_find_buddy(account, who)))
-		gf_event_info_set_buddy(info, buddy);
-
-	gf_display_show_event(info);
 }
 
 /*******************************************************************************
@@ -700,25 +561,15 @@ static void
 gf_event_account(GaimAccount *account, const gchar *warner,
 				 gint level, gpointer data)
 {
-	GfEventInfo *info;
+	GaimBuddy *buddy = NULL;
 	gchar *notification = (gchar *)data;
 
-	g_return_if_fail(data);
+	if(warner)
+		buddy = gaim_find_buddy(account, warner);
 
-	if(g_list_find(accounts, account))
-		return;
-
-	if(!gf_event_show_notification(notification))
-		return;
-
-	if(!gf_event_show_while_away(account))
-		return;
-
-	info = gf_event_info_new(notification);
-	gf_event_info_set_account(info, account);
-	gf_event_info_set_target(info, (warner) ? warner : _("Anonymous"));
-
-	gf_display_show_event(info);
+	gf_event_common(notification, account, buddy, NULL,
+					(warner) ? warner : _("Anonymous"), NULL,
+					GAIM_CBFLAGS_NONE, NULL, NULL);
 }
 
 /*******************************************************************************
@@ -764,6 +615,8 @@ gf_events_init(GaimPlugin *plugin) {
 				 _("Displayed when a topic is changed in a chat."));
 	gf_event_new("account", "warned", TOKENS_DEFAULT, _("Warned"),
 				 _("Displayed when you are warned."));
+	gf_event_new("util", GF_NOTIFICATION_MASTER, TOKENS_DEFAULT "Ccnr",
+				 _("Master"), _("Master notification for the theme editor."));
 
 	/* add our string list pref */
 	for(l = events; l; l = l->next) {
@@ -817,9 +670,9 @@ gf_events_init(GaimPlugin *plugin) {
 						GAIM_CALLBACK(gf_event_chat_message), "chat-message");
 	gaim_signal_connect(conv_handle, "received-chat-msg", plugin,
 						GAIM_CALLBACK(gf_event_chat_nick), "nick-highlight");
-	gaim_signal_connect(conv_handle, "chat-buddy-joining", plugin,
+	gaim_signal_connect(conv_handle, "chat-buddy-joined", plugin,
 						GAIM_CALLBACK(gf_event_chat_join), "chat-join");
-	gaim_signal_connect(conv_handle, "chat-buddy-leaving", plugin,
+	gaim_signal_connect(conv_handle, "chat-buddy-left", plugin,
 						GAIM_CALLBACK(gf_event_chat_part), "chat-part");
 	gaim_signal_connect(conv_handle, "chat-invited", plugin,
 						GAIM_CALLBACK(gf_event_chat_invite), "chat-invite");
