@@ -1,21 +1,21 @@
 /*
-  Guifications - The end all, be all, toaster popup plugin
-  Copyright (C) 2003-2004 Gary Kramlich
-
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License
-  as published by the Free Software Foundation; either version 2
-  of the License, or (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * Guifications - The end all, be all, toaster popup plugin
+ * Copyright (C) 2003-2004 Gary Kramlich
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 #ifdef HAVE_CONFIG_H
 # include "../gf_config.h"
 # if HAVE_UNISTD_H
@@ -46,15 +46,15 @@
 #include <util.h>
 #include <xmlnode.h>
 
-#ifdef _WIN32
-# include <win32dep.h>
-#else
+#ifndef _WIN32
 # include <gdk/gdkx.h>
 #endif
 
 #include "gf_event.h"
 #include "gf_event_info.h"
+#include "gf_gtk_utils.h"
 #include "gf_item.h"
+#include "gf_internal.h"
 #include "gf_item_text.h"
 #include "gf_notification.h"
 #include "gf_preferences.h"
@@ -185,53 +185,60 @@ text_clipping_to_string(GfItemTextClipping clip) {
 	g_return_val_if_fail(clip != GF_ITEM_TEXT_CLIPPING_UNKNOWN, NULL);
 
 	switch(clip) {
-		case GF_ITEM_TEXT_CLIPPING_TRUNCATE:		return "truncate";		break;
-		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_START:	return "ellipsis-start";	break;
-		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_MIDDLE:	return "ellipsis-middle";	break;
-		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_END:	return "ellipsis-end";	break;
+		case GF_ITEM_TEXT_CLIPPING_TRUNCATE:
+			return "truncate";
+			break;
+		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_START:
+			return "ellipsis-start";
+			break;
+		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_MIDDLE:
+			return "ellipsis-middle";
+			break;
+		case GF_ITEM_TEXT_CLIPPING_ELLIPSIS_END:
+			return "ellipsis-end";
+			break;
 		case GF_ITEM_TEXT_CLIPPING_UNKNOWN:
-		default:									return NULL;			break;
+		default:
+			return NULL;
+			break;
 	}
 }
 
 GfItemText *
 gf_item_text_new_from_xmlnode(GfItem *item, xmlnode *node) {
 	GfItemText *item_text;
-	gchar *data;
-	const gchar *width;
+	const gchar *data = NULL;
 
 	g_return_val_if_fail(item, NULL);
 	g_return_val_if_fail(node, NULL);
 
 	item_text = gf_item_text_new(item);
 
-	item_text->format = g_strdup(xmlnode_get_attrib(node, "format"));
-	if(!item_text->format) {
+	if(!(data = xmlnode_get_attrib(node, "format"))) {
 		gaim_debug_info("Guifications", "** Error loading text item: 'No format given'\n");
 		gf_item_text_destroy(item_text);
 		return NULL;
 	}
+	item_text->format = g_strdup(data);
 
-	if((data = g_strdup(xmlnode_get_attrib(node, "font")))) {
+	if((data = xmlnode_get_attrib(node, "font")))
 		item_text->font = g_strdup(data);
-		g_free(data);
-	}
 
-	if((data = g_strdup(xmlnode_get_attrib(node, "color")))) {
+	if((data = xmlnode_get_attrib(node, "color")))
 		item_text->color = g_strdup(data);
-		g_free(data);
-	}
 
-	item_text->clipping = text_clipping_from_string(xmlnode_get_attrib(node, "clipping"));
+	data = xmlnode_get_attrib(node, "clipping");
+	item_text->clipping = text_clipping_from_string(data);
 	if(item_text->clipping == GF_ITEM_TEXT_CLIPPING_UNKNOWN) {
-		gaim_debug_info("Guifications", "** Error loading text item: 'Unknown clipping type'\n");
+		gaim_debug_info("Guifications", "** Error loading text item: "
+						"'Unknown clipping type'\n");
 		gf_item_destroy(item);
 		return NULL;
 	}
 
-	width = xmlnode_get_attrib(node, "width");
-	if(width)
-		item_text->width = atoi(width);
+	data = xmlnode_get_attrib(node, "width");
+	if(data)
+		item_text->width = atoi(data);
 	else
 		item_text->width = 0;
 
@@ -258,7 +265,7 @@ gf_item_text_copy(GfItemText *text) {
 	new_text->clipping = text->clipping;
 	new_text->width = text->width;
 
-	return text;
+	return new_text;
 }
 
 xmlnode *
@@ -477,7 +484,7 @@ gf_item_text_parse_format(GfItemText *item_text, GfEventInfo *info) {
 					GaimBuddy *buddy;
 
 					buddy = gaim_find_buddy(account, conv->name);
-					if(buddy && gaim_prefs_get_bool(GF_PREF_APPEARANCE_ALIAS))
+					if(buddy)
 						str = g_string_append(str, gaim_buddy_get_contact_alias(buddy));
 					else
 						str = g_string_append(str, conv->name);
@@ -485,7 +492,7 @@ gf_item_text_parse_format(GfItemText *item_text, GfEventInfo *info) {
 					GaimChat *chat;
 
 					chat = gaim_blist_find_chat(account, conv->name);
-					if(chat && gaim_prefs_get_bool(GF_PREF_APPEARANCE_ALIAS))
+					if(chat)
 						str = g_string_append(str, gaim_chat_get_display_name(chat));
 					else
 						str = g_string_append(str, conv->name);
@@ -541,10 +548,7 @@ gf_item_text_parse_format(GfItemText *item_text, GfEventInfo *info) {
 				if(!g_ascii_strcasecmp(e_type, "buddy")) {
 					buddy = gf_event_info_get_buddy(info);
 
-					if(gaim_prefs_get_bool(GF_PREF_APPEARANCE_ALIAS))
-						str = g_string_append(str, gaim_buddy_get_contact_alias(buddy));
-					else
-						str = g_string_append(str, buddy->name);
+					str = g_string_append(str, gaim_buddy_get_contact_alias(buddy));
 				} else if(!g_ascii_strcasecmp(e_type, "conversation")) {
 					const gchar *target = gf_event_info_get_target(info);
 
@@ -553,7 +557,7 @@ gf_item_text_parse_format(GfItemText *item_text, GfEventInfo *info) {
 
 					buddy = gaim_find_buddy(account,
 									gf_event_info_get_target(info));
-					if(buddy && gaim_prefs_get_bool(GF_PREF_APPEARANCE_ALIAS))
+					if(buddy)
 						str = g_string_append(str, gaim_buddy_get_contact_alias(buddy));
 					else
 						str = g_string_append(str, gf_event_info_get_target(info));
@@ -893,6 +897,8 @@ gf_item_text_create_layout(GfItemText *item_text, GfEventInfo *info, gint width)
 		font = pango_font_description_from_string(item_text->font);
 		pango_layout_set_font_description(layout, font);
 		pango_font_description_free(font);
+	} else {
+		pango_layout_set_font_description(layout, gf_gtk_theme_get_font());
 	}
 
 	text = gf_item_text_parse_format(item_text, info);
@@ -993,7 +999,7 @@ gf_item_text_render(GfItemText *item_text, GdkPixbuf *pixbuf, GfEventInfo *info)
 	if(!layout)
 		return;
 
-/* setup the FT_BITMAP */
+	/* setup the FT_BITMAP */
 	pango_layout_get_pixel_size(layout, &l_width, &l_height);
 	bitmap.rows = l_height;
 	bitmap.width = l_width;
@@ -1004,9 +1010,12 @@ gf_item_text_render(GfItemText *item_text, GdkPixbuf *pixbuf, GfEventInfo *info)
 	pango_ft2_render_layout(&bitmap, layout, 0, 0);
 	g_object_unref(G_OBJECT(layout));
 
-	if(!item_text->color)
-		color.red = color.green = color.blue = 0;
-	else if(!pango_color_parse(&color, item_text->color))
+	if(!item_text->color) {
+		GdkColor g_color;
+
+		gf_gtk_theme_get_fg_color(&g_color);
+		gf_gtk_color_pango_from_gdk(&color, &g_color);
+	} else if(!pango_color_parse(&color, item_text->color))
 		color.red = color.green = color.blue = 0;
 
 	t_pixbuf = gf_pixbuf_new_from_ft2_bitmap(&bitmap, &color);
@@ -1021,12 +1030,7 @@ gf_item_text_render(GfItemText *item_text, GdkPixbuf *pixbuf, GfEventInfo *info)
 	gf_item_get_render_position(&x, &y, t_width, t_height, n_width, n_height,
 								item_text->item);
 
-	gdk_pixbuf_composite(t_pixbuf, pixbuf,
-						 x, y,
-						 t_width, t_height,
-						 x, y,
-						 1, 1,
-						 GDK_INTERP_BILINEAR, 255);
+	gf_gtk_pixbuf_clip_composite(t_pixbuf, x, y, pixbuf);
 
 	g_object_unref(G_OBJECT(t_pixbuf));
 }
