@@ -1,21 +1,21 @@
 /*
-  Guifications - The end all, be all, toaster popup plugin
-  Copyright (C) 2003-2004 Gary Kramlich
-
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License
-  as published by the Free Software Foundation; either version 2
-  of the License, or (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * Guifications - The end all, be all, toaster popup plugin
+ * Copyright (C) 2003-2004 Gary Kramlich
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
@@ -25,6 +25,7 @@
 #include <string.h>
 
 #include <debug.h>
+#include <notify.h>
 #include <plugin.h>
 #include <prefs.h>
 #include <request.h>
@@ -32,15 +33,11 @@
 #include <gtkutils.h>
 #include <gtkprefs.h>
 
-#ifdef _WIN32
-# include <win32dep.h>
-#endif
-
 #include "gf_action.h"
 #include "gf_display.h"
 #include "gf_event.h"
 #include "gf_file.h"
-#include "gf_intl.h"
+#include "gf_internal.h"
 #include "gf_menu.h"
 #include "gf_preferences.h"
 #include "gf_stock.h"
@@ -201,8 +198,56 @@ make_mouse_option_menu(const gchar *text, gchar *pref, GtkSizeGroup *sg) {
 
 	return hbox;
 }
+
 /*******************************************************************************
- * Behavior page stuff
+ * General page stuff
+ ******************************************************************************/
+static void
+make_general_page(GtkWidget *notebook) {
+	GtkWidget *vbox, *frame, *spin, *option, *label;
+	GtkSizeGroup *sg;
+
+	vbox = make_notebook_page(GTK_NOTEBOOK(notebook), _("General"), -1);
+	gtk_widget_show(vbox);
+
+	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+
+	frame = gaim_gtk_make_frame(vbox, _("Display Options"));
+	gtk_widget_show(frame);
+
+	option = make_option_menu(_("_Position:"), GF_PREF_APPEARANCE_POSITION, sg,
+							  gf_menu_position);
+	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
+
+	label = gaim_gtk_prefs_dropdown(frame, _("Stack:"), GAIM_PREF_BOOLEAN,
+									GF_PREF_APPEARANCE_VERTICAL,
+									_("Vertically"), TRUE,
+									_("Horizontally"), FALSE,
+									NULL);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+	gtk_size_group_add_widget(sg, label);
+
+	spin = gaim_gtk_prefs_labeled_spin_button(frame, _("_Display Time:"),
+											  GF_PREF_BEHAVIOR_DISPLAY_TIME,
+											  1, 60, sg);
+	label = make_label(_("seconds"), NULL);
+	gtk_box_pack_start(GTK_BOX(spin), label, FALSE, FALSE, 0);
+
+	frame = gaim_gtk_make_frame(vbox, _("Mouse"));
+	gtk_widget_show(frame);
+
+	option = make_mouse_option_menu(_("Left:"), GF_PREF_MOUSE_LEFT, sg);
+	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
+
+	option = make_mouse_option_menu(_("Middle:"), GF_PREF_MOUSE_MIDDLE, sg);
+	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
+
+	option = make_mouse_option_menu(_("Right:"), GF_PREF_MOUSE_RIGHT, sg);
+	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
+}
+
+/*******************************************************************************
+ * Notification page stuff
  ******************************************************************************/
 static gint
 notification_sort_show(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
@@ -324,15 +369,18 @@ make_notification_list(GtkBox *parent) {
 		const gchar *type;
 
 		event = GF_EVENT(events->data);
+		type = gf_event_get_notification_type(event);
+
+		if(type && type[0] == '!')
+			continue;
 
 		gtk_list_store_append(store, &iter);
 
-		type = gf_event_get_notification_type(event);
 		gtk_list_store_set(store, &iter,
 						   GF_NOTIF_COL_SHOW, gf_event_show_notification(type),
 						   GF_NOTIF_COL_NAME, gf_event_get_name(event),
 						   GF_NOTIF_COL_DESCRIPTION, gf_event_get_description(event),
-						   GF_NOTIF_COL_TYPE, gf_event_get_notification_type(event),
+						   GF_NOTIF_COL_TYPE, type,
 						   -1);
 	}
 
@@ -375,123 +423,13 @@ make_notification_list(GtkBox *parent) {
 }
 
 static void
-make_behavior_page(GtkWidget *notebook) {
-	GtkWidget *vbox, *frame;
-	GtkWidget *label, *spin;
-	GtkSizeGroup *sg;
+make_notifications_page(GtkWidget *notebook) {
+	GtkWidget *vbox;
 
-	vbox = make_notebook_page(GTK_NOTEBOOK(notebook), _("Behavior"), -1);
+	vbox = make_notebook_page(GTK_NOTEBOOK(notebook), _("Notifications"), -1);
 	gtk_widget_show(vbox);
 
-	frame = gaim_gtk_make_frame(vbox, _("Notifications"));
-
-	/* dirty nasty hack to make better use of the behavior page */
-	gtk_box_set_child_packing(GTK_BOX(frame->parent), frame, TRUE, TRUE, 0,
-							  GTK_PACK_START);
-	gtk_box_set_child_packing(GTK_BOX(frame->parent->parent), frame->parent,
-							  TRUE, TRUE, 0, GTK_PACK_START);
-	gtk_box_set_child_packing(GTK_BOX(frame->parent->parent->parent),
-							  frame->parent->parent, TRUE, TRUE, 0,
-							  GTK_PACK_START);
-
-	make_notification_list(GTK_BOX(frame));
-
-	gtk_widget_show(frame);
-
-	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-
-	frame = gaim_gtk_make_frame(vbox, _("Display Options"));
-	gtk_widget_show(frame);
-
-	spin = gaim_gtk_prefs_labeled_spin_button(frame, _("_Display Time:"),
-											  GF_PREF_BEHAVIOR_DISPLAY_TIME,
-											  1, 60, sg);
-	label = make_label(_("seconds"), NULL);
-	gtk_box_pack_start(GTK_BOX(spin), label, FALSE, FALSE, 0);
-
-	gaim_gtk_prefs_labeled_spin_button(frame, _("_Max Visible Guifications:"),
-									   GF_PREF_BEHAVIOR_THROTTLE,
-									   0, 255, sg);
-
-	gaim_gtk_prefs_checkbox(_("_Show Guifications while away"),
-							GF_PREF_BEHAVIOR_SHOW_WHILE_AWAY,
-							frame);
-
-	gaim_gtk_prefs_checkbox(_("_Show Guifications while screensaver is active"),
-							GF_PREF_BEHAVIOR_SHOW_DURING_SCREEN_SAVER,
-							frame);
-}
-
-/*******************************************************************************
- * Appearance page stuff
- ******************************************************************************/
-static void
-make_appearance_page(GtkWidget *notebook) {
-	GtkWidget *vbox, *frame, *label, *option;
-	GtkSizeGroup *sg;
-
-	vbox = make_notebook_page(GTK_NOTEBOOK(notebook), _("Appearance"), -1);
-	gtk_widget_show(vbox);
-
-	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-
-	frame = gaim_gtk_make_frame(vbox, _("Position Options"));
-	gtk_widget_show(frame);
-
-	option = make_option_menu(_("_Position:"), GF_PREF_APPEARANCE_POSITION, sg,
-							  gf_menu_position);
-	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
-
-	label = gaim_gtk_prefs_dropdown(frame, _("Stack:"), GAIM_PREF_BOOLEAN,
-									GF_PREF_APPEARANCE_VERTICAL,
-									_("Vertically"), TRUE,
-									_("Horizontally"), FALSE,
-									NULL);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-	gtk_size_group_add_widget(sg, label);
-
-	frame = gaim_gtk_make_frame(vbox, _("Display Options"));
-	gtk_widget_show(frame);
-
-	option = make_option_menu(_("_Zoom:"), GF_PREF_APPEARANCE_ZOOM, sg,
-							  gf_menu_zoom);
-	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
-
-	gaim_gtk_prefs_checkbox(_("Show _alias if set"), GF_PREF_APPEARANCE_ALIAS,
-							frame);
-
-	/* Change this to "Condense notification for the same contact" or something along
-	 * those lines
-	 */
-	gaim_gtk_prefs_checkbox(_("Show _contacts as one guification"),
-							GF_PREF_APPEARANCE_CONTACTS, frame);
-}
-
-/*******************************************************************************
- * Mouse page stuff
- ******************************************************************************/
-static void
-make_mouse_page(GtkWidget *notebook) {
-	GtkWidget *vbox, *frame;
-	GtkWidget *option;
-	GtkSizeGroup *sg;
-
-	vbox = make_notebook_page(GTK_NOTEBOOK(notebook), _("Mouse"), -1);
-	gtk_widget_show(vbox);
-
-	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-
-	frame = gaim_gtk_make_frame(vbox, _("Buttons:"));
-	gtk_widget_show(frame);
-
-	option = make_mouse_option_menu(_("Left:"), GF_PREF_MOUSE_LEFT, sg);
-	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
-
-	option = make_mouse_option_menu(_("Middle:"), GF_PREF_MOUSE_MIDDLE, sg);
-	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
-
-	option = make_mouse_option_menu(_("Right:"), GF_PREF_MOUSE_RIGHT, sg);
-	gtk_box_pack_start(GTK_BOX(frame), option, FALSE, FALSE, 0);
+	make_notification_list(GTK_BOX(vbox));
 }
 
 /*******************************************************************************
@@ -505,6 +443,7 @@ struct GfThemeListData {
 	GtkWidget *theme_delete;
 	GtkWidget *theme_copy;
 	GtkWidget *theme_refresh;
+	GtkWidget *theme_get_more;
 };
 
 struct ThemeInfoPane {
@@ -660,7 +599,7 @@ theme_got_url(void *data, const char *themedata, size_t len)
 
 	theme_install_theme(path, data);
 
-	unlink(path);
+	g_unlink(path);
 	g_free(path);
 }
 
@@ -961,8 +900,8 @@ theme_list_delete_cb(GtkWidget *w, gpointer data) {
 	if(!filename)
 		return;
 
-	gaim_request_yes_no(guifications, "Delete theme?", NULL,
-						"Are you sure you want to delete this theme?",
+	gaim_request_yes_no(guifications, _("Delete theme?"), NULL,
+						_("Are you sure you want to delete this theme?"),
 						0, filename,
 						G_CALLBACK(theme_list_delete_yes_cb),
 						G_CALLBACK(theme_list_delete_no_cb));
@@ -1026,6 +965,11 @@ theme_list_copy_cb(GtkWidget *w, gpointer data) {
 	theme_list_refresh();
 }
 
+static void
+theme_list_get_more_cb(GtkWidget *w, gpointer data) {
+	gaim_notify_uri(NULL, "http://sourceforge.net/tracker/?atid=676821&group_id=92888&func=browse");
+}
+
 static gboolean
 theme_list_clicked_cb(GtkWidget *w, GdkEventButton *e, gpointer data) {
 	if(e->button == 3) {
@@ -1132,7 +1076,7 @@ make_theme_list(GtkBox *parent) {
 	g_signal_connect(G_OBJECT(sel), "changed",
 					 G_CALLBACK(theme_list_selection_cb), NULL);
 
-	/* the new, edit, delete, refresh buttons */
+	/* the new, edit, delete, refresh buttons and get more buttons */
 	hbox = gtk_hbox_new(FALSE, 4);
 	gtk_box_pack_start(GTK_BOX(parent), hbox, FALSE, FALSE, 0);
 
@@ -1172,6 +1116,15 @@ make_theme_list(GtkBox *parent) {
 					 G_CALLBACK(theme_list_refresh_cb), NULL);
 	gtk_box_pack_start(GTK_BOX(hbox), theme_data.theme_refresh, FALSE, FALSE, 0);
 
+	theme_data.theme_get_more = 
+					gaim_pixbuf_button_from_stock(_("_Get More"),
+												  GTK_STOCK_JUMP_TO,
+												  GAIM_BUTTON_HORIZONTAL);
+	gtk_button_set_relief(GTK_BUTTON(theme_data.theme_get_more), GTK_RELIEF_NONE);
+	g_signal_connect(G_OBJECT(theme_data.theme_get_more), "clicked",
+					 G_CALLBACK(theme_list_get_more_cb), NULL);
+	gtk_box_pack_start(GTK_BOX(hbox), theme_data.theme_get_more, FALSE, FALSE, 0);
+	
 	gtk_widget_show_all(sw);
 }
 
@@ -1281,8 +1234,11 @@ make_theme_page(GtkWidget *notebook) {
  ******************************************************************************/
 static void
 make_advanced_page(GtkWidget *notebook) {
-	GtkWidget *vbox, *frame, *hbox, *label;
+	GtkWidget *vbox, *frame;
 	GtkSizeGroup *sg;
+#if GTK_CHECK_VERSION(2,2,0)
+	int screen_count, monitor_count;
+#endif
 
 	sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -1296,31 +1252,29 @@ make_advanced_page(GtkWidget *notebook) {
 
 	frame = gaim_gtk_make_frame(vbox, _("Display Options"));
 
-	hbox = gaim_gtk_prefs_labeled_spin_button(frame, _("Sign on delay:"),
-											  GF_PREF_ADVANCED_SIGN_ON_DELAY,
-											  0, 30000, sg);
-	label = gtk_label_new(_("milliseconds"));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
-
-	frame = gaim_gtk_make_frame(vbox, _("Conversations"));
-
-	gaim_gtk_prefs_checkbox(_("Hide notifications for focused conversations"),
-							GF_PREF_ADVANCED_HIDE_CONV_FOCUSED,
-							frame);
-
-	gaim_gtk_prefs_checkbox(_("Hide chat join / leave messages"),
-							GF_PREF_ADVANCED_HIDE_JOINLEAVE,
-							frame);
+	gaim_gtk_prefs_labeled_spin_button(frame, _("_Max Visible Guifications:"),
+									   GF_PREF_BEHAVIOR_THROTTLE,
+									   0, 255, sg);
 
 #if GTK_CHECK_VERSION(2,2,0)
-	frame = gaim_gtk_make_frame(vbox, _("Placement"));
+	screen_count = gf_display_get_screen_count();
+	monitor_count = gf_display_get_monitor_count();
+	if(screen_count > 0 || monitor_count > 0) {
+		frame = gaim_gtk_make_frame(vbox, _("Placement"));
 
-	gaim_gtk_prefs_labeled_spin_button(frame, _("Show notifications on screen:"),
-									   GF_PREF_ADVANCED_SCREEN,
-									   0, gf_display_get_screen_count(), sg);
-	gaim_gtk_prefs_labeled_spin_button(frame, _("Show notifications on monitor:"),
-									   GF_PREF_ADVANCED_MONITOR,
-									   0, gf_display_get_monitor_count(), sg);
+		if(screen_count > 0) {
+			gaim_gtk_prefs_labeled_spin_button(frame,
+										_("Show notifications on screen:"),
+										GF_PREF_ADVANCED_SCREEN,
+										0, screen_count, sg);
+		}
+		if(monitor_count > 0) {
+			gaim_gtk_prefs_labeled_spin_button(frame,
+										_("Show notifications on monitor:"),
+										GF_PREF_ADVANCED_MONITOR,
+										0, monitor_count, sg);
+		}
+	}
 #endif /* GTK_CHECK_VERSION(2,2,0) */
 
 	gtk_widget_show_all(vbox);
@@ -1345,9 +1299,8 @@ gf_preferences_get_frame(GaimPlugin *plugin) {
 					 G_CALLBACK(gf_preferences_notebook_destroy_cb), NULL);
 	gtk_widget_show(notebook);
 
-	make_behavior_page(notebook);
-	make_appearance_page(notebook);
-	make_mouse_page(notebook);
+	make_general_page(notebook);
+	make_notifications_page(notebook);
 	make_theme_page(notebook);
 	make_advanced_page(notebook);
 
@@ -1370,15 +1323,10 @@ gf_preferences_add() {
 	gaim_prefs_add_none(GF_PREF_BEHAVIOR_ROOT);
 	gaim_prefs_add_int(GF_PREF_BEHAVIOR_DISPLAY_TIME, 6);
 	gaim_prefs_add_int(GF_PREF_BEHAVIOR_THROTTLE, 6);
-	gaim_prefs_add_bool(GF_PREF_BEHAVIOR_SHOW_WHILE_AWAY, TRUE);
-	gaim_prefs_add_bool(GF_PREF_BEHAVIOR_SHOW_DURING_SCREEN_SAVER, TRUE);
 
 	gaim_prefs_add_none(GF_PREF_APPEARANCE_ROOT);
 	gaim_prefs_add_int(GF_PREF_APPEARANCE_POSITION, GF_DISPLAY_POSITION_SE);
 	gaim_prefs_add_bool(GF_PREF_APPEARANCE_VERTICAL, TRUE);
-	gaim_prefs_add_int(GF_PREF_APPEARANCE_ZOOM, GF_DISPLAY_ZOOM_100);
-	gaim_prefs_add_bool(GF_PREF_APPEARANCE_ALIAS, TRUE);
-	gaim_prefs_add_bool(GF_PREF_APPEARANCE_CONTACTS, TRUE);
 
 	gaim_prefs_add_none(GF_PREF_MOUSE_ROOT);
 	gaim_prefs_add_string(GF_PREF_MOUSE_LEFT, "open");
@@ -1395,18 +1343,35 @@ gf_preferences_add() {
 	gaim_prefs_add_none(GF_PREF_ADVANCED_ROOT);
 	gaim_prefs_add_bool(GF_PREF_ADVANCED_RELEASE_NOTIFICATION, TRUE);
 	gaim_prefs_add_int(GF_PREF_ADVANCED_RELEASE_LAST_CHECK, 0);
-	gaim_prefs_add_int(GF_PREF_ADVANCED_SIGN_ON_DELAY, 8000);
-	gaim_prefs_add_bool(GF_PREF_ADVANCED_HIDE_CONV_FOCUSED, FALSE);
-	gaim_prefs_add_bool(GF_PREF_ADVANCED_HIDE_JOINLEAVE, FALSE);
 
 #if GTK_CHECK_VERSION(2,2,0)
 	gaim_prefs_add_int(GF_PREF_ADVANCED_SCREEN, 0);
 	gaim_prefs_add_int(GF_PREF_ADVANCED_MONITOR, 0);
+
+	if(gaim_prefs_get_int(GF_PREF_ADVANCED_SCREEN) >
+	   gf_display_get_screen_count())
+	{
+		gaim_prefs_set_int(GF_PREF_ADVANCED_SCREEN,
+						   gf_display_get_default_screen());
+	}
+
+	if(gaim_prefs_get_int(GF_PREF_ADVANCED_MONITOR) >
+	   gf_display_get_monitor_count())
+	{
+		gaim_prefs_set_int(GF_PREF_ADVANCED_MONITOR,
+						   gf_display_get_default_monitor());
+	}
 #endif /* GTK_CHECK_VERSION(2,2,0) */
 
 	/* clean up old prefs if they exist */
-	if(gaim_prefs_exists("/plugins/gtk/amc_grim/guifications2/appearance/h_offset"))
-		gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/h_offset");
-	if(gaim_prefs_exists("/plugins/gtk/amc_grim/guifications2/appearance/v_offset"))
-		gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/h_offset");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/behavior/show_during_screen_saver");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/behavior/show_while_away");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/h_offset");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/h_offset");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/zoom");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/show_contacts");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/appearance/alias");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/advanced/sign_on_delay");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/advanced/hide_joinleave");
+	gaim_prefs_remove("/plugins/gtk/amc_grim/guifications2/advanced/hide_conv_focused");
 }
