@@ -1,21 +1,21 @@
 /*
-  Guifications - The end all, be all, toaster popup plugin
-  Copyright (C) 2003-2004 Gary Kramlich
-
-  This program is free software; you can redistribute it and/or
-  modify it under the terms of the GNU General Public License
-  as published by the Free Software Foundation; either version 2
-  of the License, or (at your option) any later version.
-
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/
+ * Guifications - The end all, be all, toaster popup plugin
+ * Copyright (C) 2003-2004 Gary Kramlich
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 #ifdef HAVE_CONFIG_H
 # include "../gf_config.h"
 #endif
@@ -28,10 +28,6 @@
 
 #define GAIM_PLUGINS
 
-#ifdef _WIN32
-# include <win32dep.h>
-#endif
-
 #include <debug.h>
 #include <gaim.h>
 #include <notify.h>
@@ -43,7 +39,7 @@
 
 #include "gf_event.h"
 #include "gf_file.h"
-#include "gf_intl.h"
+#include "gf_internal.h"
 #include "gf_item.h"
 #include "gf_item_icon.h"
 #include "gf_item_image.h"
@@ -126,8 +122,11 @@ struct GfThemeEditorOps {
 };
 
 struct GfThemeEditorNotification {
+	GtkWidget *use_gtk;
 	GtkWidget *filename;
 	GtkWidget *button;
+	GtkWidget *width;
+	GtkWidget *height;
 };
 
 struct GfThemeEditorItem {
@@ -161,6 +160,7 @@ struct GfThemeEditorText {
 
 struct GfThemeEditor {
 	GfTheme *theme;
+	GfNotification *master;
 
 	gchar *filename;
 	gchar *path;
@@ -172,6 +172,8 @@ struct GfThemeEditor {
 	GtkWidget *tool_item;
 	GtkWidget *tool_copy;
 	GtkWidget *tool_delete;
+	GtkWidget *tool_up;
+	GtkWidget *tool_down;
 
 	/* main stuff */
 	GtkWidget *window;
@@ -213,6 +215,11 @@ struct GfteNewItem {
 typedef void (*GfteSetFunc)(gpointer obj, gconstpointer val);
 typedef gconstpointer (*GfteGetFunc)(gpointer obj);
 
+/*******************************************************************************
+ * Prototypes ICK!
+ ******************************************************************************/
+static void gfte_selection_changed_cb(GtkTreeSelection *sel, gpointer data);
+
 /*******************************************************************************
  * Globals
  ******************************************************************************/
@@ -229,10 +236,14 @@ struct GfteNewItem new_item;
  * Toolbar button stuff
  ******************************************************************************/
 static void
-gfte_toolbar_buttons_update(gboolean item, gboolean copy, gboolean delete) {
+gfte_toolbar_buttons_update(gboolean item, gboolean copy, gboolean delete,
+							gboolean up, gboolean down)
+{
 	gtk_widget_set_sensitive(editor.tool_item, item);
 	gtk_widget_set_sensitive(editor.tool_copy, copy);
 	gtk_widget_set_sensitive(editor.tool_delete, delete);
+	gtk_widget_set_sensitive(editor.tool_up, up);
+	gtk_widget_set_sensitive(editor.tool_down, down);
 }
 
 /*******************************************************************************
@@ -254,6 +265,16 @@ gfte_remove_temp() {
 static void
 gfte_save_theme() {
 	gchar *path, *oldpath, *dir;
+	gboolean loaded;
+
+	loaded = gf_theme_is_loaded(editor.filename);
+
+	if(loaded) {
+		GfTheme *theme = gf_theme_find_theme_by_filename(editor.filename);
+
+		if(theme)
+			gf_theme_unload(theme);
+	}
 
 	gf_theme_save_to_file(editor.theme, editor.filename);
 
@@ -268,18 +289,22 @@ gfte_save_theme() {
 	g_free(dir);
 
 	/* move the dir */
-	rename(editor.path, path);
+	g_rename(editor.path, path);
 
 	/* update the editor's path */
 	g_free(editor.path);
 	editor.path = path;
 
-	/* update the editor's file */
+    /* update the editor's file */
 	g_free(editor.filename);
 	editor.filename = g_build_filename(editor.path, "theme.xml", NULL);
 
 	/* reprobe the theme and tell the prefs to refresh it.. */
 	gf_theme_probe(editor.filename);
+	if(loaded) {
+		gf_theme_load(editor.filename);
+		gf_themes_save_loaded();
+	}
 	gf_preferences_refresh_themes_list();
 
 	editor.changed = FALSE;
@@ -419,7 +444,7 @@ gfte_make_temp_dir() {
 	path = g_build_filename(gaim_user_dir(), "guifications", "themes", dir, NULL);
 	g_free(dir);
 
-	mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR);
+	g_mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR);
 
 	return path;
 }
@@ -429,11 +454,28 @@ gfte_setup(const gchar *filename) {
 	GfTheme *old = editor.theme;
 
 	if(filename) {
+		GfNotification *notification;
+		GList *l;
+
 		editor.theme = gf_theme_new_from_file(filename);
+
+		for(l = gf_theme_get_notifications(editor.theme); l; l = l->next) {
+			notification = GF_NOTIFICATION(l->data);
+
+			if(!g_ascii_strcasecmp(GF_NOTIFICATION_MASTER,
+								   gf_notification_get_type(notification))) {
+				editor.master = notification;
+				break;
+			}
+		}
 	} else {
 		editor.theme = gf_theme_new();
 		gf_theme_set_theme_info(editor.theme, gf_theme_info_new());
 		gf_theme_set_theme_options(editor.theme, gf_theme_options_new());
+
+		editor.master = gf_notification_new(editor.theme);
+		gf_notification_set_type(editor.master, GF_NOTIFICATION_MASTER);
+		gf_theme_add_notification(editor.theme, editor.master);
 	}
 
 	if(editor.theme) {
@@ -783,7 +825,7 @@ gfte_delete_yes_cb(GtkButton *button, gpointer data) {
 	gtk_widget_destroy(del_obj.window);
 	del_obj.window = NULL;
 
-	gfte_toolbar_buttons_update(FALSE, FALSE, FALSE);
+	gfte_toolbar_buttons_update(FALSE, FALSE, FALSE, FALSE, FALSE);
 
 	editor.changed = TRUE;
 }
@@ -896,7 +938,11 @@ gfte_new_notification_ok_cb(GtkButton *button, gpointer data) {
 		return;
 	}
 
-	notification = gf_notification_new(GF_THEME(object));
+	if(!editor.master) {
+		notification = gf_notification_new(GF_THEME(object));
+	} else {
+		notification = gf_notification_copy(editor.master);
+	}
 
 	/* find the notification type, this really needs to get cleaned up but works for now
 	 */
@@ -916,6 +962,32 @@ gfte_new_notification_ok_cb(GtkButton *button, gpointer data) {
 				   notification);
 	gfte_store_select_iter(&child);
 
+	if(editor.master) {
+		GtkTreeIter item;
+		GList *l;
+		gboolean expand = FALSE;
+
+		for(l = gf_notification_get_items(notification); l; l = l->next) {
+			gint type = gf_item_get_type(GF_ITEM(l->data));
+
+			gfte_store_add(editor.store, &item, &child,
+						   gf_item_type_to_string(type, TRUE),
+						   type + GFTE_PAGE_ICON,
+						   l->data);
+
+			if(!expand)
+				expand = TRUE;
+		}
+
+		if(expand) {
+			GtkTreePath *path;
+
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(editor.store), &child);
+			gtk_tree_view_expand_row(GTK_TREE_VIEW(editor.tree), path, TRUE);
+			gtk_tree_path_free(path);
+		}		
+	}
+
 	if(new_notification.window)
 		gtk_widget_destroy(new_notification.window);
 	new_notification.window = NULL;
@@ -1151,6 +1223,7 @@ gfte_duplicate_object(GtkButton *button, gpointer data) {
 		GfNotification *notification;
 		GtkTreeIter item;
 		GList *l;
+		gboolean expand = FALSE;
 
 		notification = gf_notification_copy(GF_NOTIFICATION(object));
 
@@ -1164,9 +1237,20 @@ gfte_duplicate_object(GtkButton *button, gpointer data) {
 						   gf_item_type_to_string(type, TRUE),
 						   type + GFTE_PAGE_ICON,
 						   l->data);
+
+			if(!expand)
+				expand = TRUE;
 		}
 
 		gf_theme_add_notification(editor.theme, notification);
+
+		if(expand) {
+			GtkTreePath *path;
+
+			path = gtk_tree_model_get_path(GTK_TREE_MODEL(editor.store), &child);
+			gtk_tree_view_expand_row(GTK_TREE_VIEW(editor.tree), path, TRUE);
+			gtk_tree_path_free(path);
+		}
 	} else if(page == GFTE_PAGE_ICON || page == GFTE_PAGE_IMAGE ||
 			  page == GFTE_PAGE_TEXT)
 	{
@@ -1175,10 +1259,10 @@ gfte_duplicate_object(GtkButton *button, gpointer data) {
 
 		gtk_tree_store_append(editor.store, &child, &parent);
 		gtk_tree_store_set(editor.store, &child,
-					   GFTE_STORE_OBJECT, item,
-					   GFTE_STORE_PAGE, page,
-					   GFTE_STORE_TITLE, title,
-					   -1);
+						   GFTE_STORE_OBJECT, item,
+						   GFTE_STORE_PAGE, page,
+						   GFTE_STORE_TITLE, title,
+						   -1);
 
 		gf_notification_add_item(gf_item_get_notification(GF_ITEM(object)),
 								 item);
@@ -1194,6 +1278,205 @@ gfte_duplicate_object(GtkButton *button, gpointer data) {
 	editor.changed = TRUE;
 }
 
+/*******************************************************************************
+ * Moving items
+ ******************************************************************************/
+static gboolean
+gfte_is_older_notification(GfNotification *notification) {
+	GfTheme *theme;
+	GList *l;
+
+	theme = gf_notification_get_theme(notification);
+	if(!theme)
+		return FALSE;
+
+	l = gf_theme_get_notifications(theme);
+	if(l->data == notification)
+		return FALSE;
+	else
+		return TRUE;
+}
+
+static gboolean
+gfte_is_younger_notification(GfNotification *notification) {
+	GfTheme *theme;
+	GList *l;
+
+	theme = gf_notification_get_theme(notification);
+	if(!theme)
+		return FALSE;
+
+	/* get the last notification */
+	for(l = gf_theme_get_notifications(theme); l->next; l = l->next);
+
+	if(l->data == notification)
+		return FALSE;
+	else
+		return TRUE;
+}
+
+static gboolean
+gfte_is_older_item(GfItem *item) {
+	GfNotification *notification;
+	GList *l;
+
+	notification = gf_item_get_notification(item);
+	if(!notification)
+		return FALSE;
+
+	l = gf_notification_get_items(notification);
+	if(l->data == item)
+		return FALSE;
+	else
+		return TRUE;
+}
+
+static gboolean
+gfte_is_younger_item(GfItem *item) {
+	GfNotification *notification;
+	GList *l;
+
+	notification = gf_item_get_notification(item);
+	if(!notification)
+		return FALSE;
+
+	/* get the last item */
+	for(l = gf_notification_get_items(notification); l->next; l = l->next);
+
+	if(l->data == item)
+		return FALSE;
+	else
+		return TRUE;
+}
+
+static void
+gfte_store_swap(GtkTreeIter *a, GtkTreeIter *b) {
+	GtkTreeSelection *sel;
+	gchar *a_title, *b_title;
+	gint a_page, b_page;
+	gpointer a_obj, b_obj;
+#if !GTK_CHECK_VERSION(2,2,0)
+	gchar *title;
+	gint page;
+	gpointer obj;
+#endif /* !GTK_CHECK_VERSION(2,2,0) */
+
+	/* get 'em */
+	gtk_tree_model_get(GTK_TREE_MODEL(editor.store), a,
+					   GFTE_STORE_OBJECT, &a_obj,
+					   GFTE_STORE_PAGE, &a_page,
+					   GFTE_STORE_TITLE, &a_title,
+					   -1);
+	gtk_tree_model_get(GTK_TREE_MODEL(editor.store), b,
+					   GFTE_STORE_OBJECT, &b_obj,
+					   GFTE_STORE_PAGE, &b_page,
+					   GFTE_STORE_TITLE, &b_title,
+					   -1);
+
+	if(a_page == GFTE_PAGE_NOTIFICATION)
+		gf_notifications_swap(GF_NOTIFICATION(a_obj), GF_NOTIFICATION(b_obj));
+	else {
+		gf_items_swap(GF_ITEM(a_obj), GF_ITEM(b_obj));
+	}
+
+#if GTK_CHECK_VERSION(2,2,0)
+	gtk_tree_store_swap(editor.store, a, b);
+#else
+	/* swap 'em */
+	title = a_title;
+	page = a_page;
+	obj = a_obj;
+
+	a_title = b_title;
+	a_page = b_page;
+	a_obj = b_obj;
+
+	b_title = title;
+	b_page = page;
+	b_obj = obj;
+
+	/* set 'em */
+	gtk_tree_store_set(editor.store, a,
+					   GFTE_STORE_OBJECT, a_obj,
+					   GFTE_STORE_PAGE, a_page,
+					   GFTE_STORE_TITLE, a_title,
+					   -1);
+	gtk_tree_store_set(editor.store, b,
+					   GFTE_STORE_OBJECT, b_obj,
+					   GFTE_STORE_PAGE, b_page,
+					   GFTE_STORE_TITLE, b_title,
+					   -1);
+
+#endif /* GTK_CHECK_VERSION(2,2,0) */
+
+	/* clean up */
+	g_free(a_title);
+	g_free(b_title);
+
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(editor.tree));
+	gfte_selection_changed_cb(sel, NULL);
+}
+
+static void
+gfte_move_up(GtkButton *button, gpointer data) {
+	GtkTreeIter child, sibling;
+	GtkTreePath *path;
+	gpointer object;
+	gchar *title;
+	gint page;
+
+	object = gfte_store_get_row(&child, &page, &title);
+
+	if(title)
+		g_free(title);
+
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(editor.store), &child);
+	if(!path)
+		return;
+
+	if(!gtk_tree_path_prev(path)) {
+		gtk_tree_path_free(path);
+		return;
+	}
+
+	if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(editor.store), &sibling, path)) {
+		gtk_tree_path_free(path);
+		return;
+	}
+
+	gfte_store_swap(&child, &sibling);
+	gtk_tree_path_free(path);
+}
+
+static void
+gfte_move_down(GtkButton *button, gpointer data) {
+	GtkTreeIter child, sibling;
+	GtkTreePath *path;
+	gpointer object;
+	gchar *title;
+	gint page;
+
+	object = gfte_store_get_row(&child, &page, &title);
+
+	if(title)
+		g_free(title);
+
+	path = gtk_tree_model_get_path(GTK_TREE_MODEL(editor.store), &child);
+	if(!path)
+		return;
+
+	/* we just assume this works since it returns void I guess.. */
+	gtk_tree_path_next(path);
+
+	if(!gtk_tree_model_get_iter(GTK_TREE_MODEL(editor.store), &sibling, path)) {
+		gtk_tree_path_free(path);
+		return;
+	}
+
+	gfte_store_swap(&child, &sibling);
+	gtk_tree_path_free(path);
+}
+
 /*******************************************************************************
  * getter/setter helpers
  ******************************************************************************/
@@ -1204,7 +1487,7 @@ gfte_store_get_object() {
 	GtkTreeSelection *sel;
 	gpointer object;
 
-	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(editor.tree));
+    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(editor.tree));
 	gtk_tree_selection_get_selected(sel, &model, &iter);
 
 	gtk_tree_model_get(model, &iter,
@@ -1314,12 +1597,23 @@ gfte_entry_changed_cb(GtkWidget *widget, gpointer data) {
 static void
 gfte_check_toggled_cb(GtkWidget *widget, gpointer data) {
 	gpointer object = gfte_store_get_object();
-	gpointer value = GINT_TO_POINTER(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
+	gpointer pvalue;
 	gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(editor.note));
+	gboolean value;
+
+	value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+	pvalue = GINT_TO_POINTER(value);
 
 	editor.changed = TRUE;
 
-	gfte_set_value(widget, page, object, value);
+	gfte_set_value(widget, page, object, pvalue);
+
+	/* adjust the sensitivity according to the use gtk setting */
+	if(widget == editor.notification.use_gtk) {
+		gtk_widget_set_sensitive(editor.notification.button, !value);
+		gtk_widget_set_sensitive(editor.notification.width, value);
+		gtk_widget_set_sensitive(editor.notification.height, value);
+	}
 }
 
 static void
@@ -1461,7 +1755,7 @@ gfte_update_entry(GtkWidget *entry, gint page, gpointer object) {
 
 static void
 gfte_update_option_menu(GtkWidget *opt_menu, gint page, gpointer object) {
-	gint value = (gint)gfte_get_value(opt_menu, page, object);
+	gint value = GPOINTER_TO_INT(gfte_get_value(opt_menu, page, object));
 
 	g_signal_handlers_block_by_func(G_OBJECT(opt_menu),
 									gfte_option_menu_changed_cb, NULL);
@@ -1474,7 +1768,7 @@ gfte_update_option_menu(GtkWidget *opt_menu, gint page, gpointer object) {
 
 static void
 gfte_update_spin_button(GtkWidget *spin, gint page, gpointer object) {
-	gint value = (gint)gfte_get_value(spin, page, object);
+	gint value = GPOINTER_TO_INT(gfte_get_value(spin, page, object));
 
 	g_signal_handlers_block_by_func(G_OBJECT(spin),
 									gfte_spin_changed_cb, NULL);
@@ -1487,7 +1781,7 @@ gfte_update_spin_button(GtkWidget *spin, gint page, gpointer object) {
 
 static void
 gfte_update_check(GtkWidget *check, gint page, gpointer object) {
-	gboolean value = (gboolean)gfte_get_value(check, page, object);
+	gboolean value = GPOINTER_TO_INT(gfte_get_value(check, page, object));
 
 	g_signal_handlers_block_by_func(G_OBJECT(check),
 									gfte_check_toggled_cb, NULL);
@@ -1496,6 +1790,13 @@ gfte_update_check(GtkWidget *check, gint page, gpointer object) {
 
 	g_signal_handlers_unblock_by_func(G_OBJECT(check),
 									  gfte_check_toggled_cb, NULL);
+
+	/* adjust the sensitivity according to the use gtk setting */
+	if(check == editor.notification.use_gtk) {
+		gtk_widget_set_sensitive(editor.notification.button, !value);
+		gtk_widget_set_sensitive(editor.notification.width, value);
+		gtk_widget_set_sensitive(editor.notification.height, value);
+	}
 }
 
 /*******************************************************************************
@@ -1702,8 +2003,14 @@ static void
 gfte_update_notification_page() {
 	GfNotification *notification = GF_NOTIFICATION(gfte_store_get_object());
 
+	gfte_update_check(editor.notification.use_gtk, GFTE_PAGE_NOTIFICATION,
+					  notification);
 	gfte_update_entry(editor.notification.filename, GFTE_PAGE_NOTIFICATION,
 					  notification);
+	gfte_update_spin_button(editor.notification.width, GFTE_PAGE_NOTIFICATION,
+							notification);
+	gfte_update_spin_button(editor.notification.height, GFTE_PAGE_NOTIFICATION,
+							notification);
 }
 
 static void
@@ -1743,6 +2050,7 @@ static void
 gfte_selection_changed_cb(GtkTreeSelection *sel, gpointer data) {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
+	gpointer object;
 	gint page;
 
 	gfte_dialog_cleanup();
@@ -1755,35 +2063,51 @@ gfte_selection_changed_cb(GtkTreeSelection *sel, gpointer data) {
 
 	gtk_tree_model_get(model, &iter,
 					   GFTE_STORE_PAGE, &page,
+					   GFTE_STORE_OBJECT, &object,
 					   -1);
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(editor.note), page);
 
 	switch(page) {
 		case GFTE_PAGE_THEME:
-			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE);
+			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE, FALSE, FALSE);
 			break;
 		case GFTE_PAGE_INFO:
-			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE);
+			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE, FALSE, FALSE);
 			gfte_update_info_page();
 			break;
 		case GFTE_PAGE_OPS:
-			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE);
+			gfte_toolbar_buttons_update(FALSE, FALSE, FALSE, FALSE, FALSE);
 			gfte_update_ops_page();
 			break;
-		case GFTE_PAGE_NOTIFICATION:
-			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE);
+		case GFTE_PAGE_NOTIFICATION: {
+			GfNotification *notification = GF_NOTIFICATION(object);
+			gboolean master;
+
+			master = (g_ascii_strcasecmp(GF_NOTIFICATION_MASTER,
+										 gf_notification_get_type(notification)));
+
+			gfte_toolbar_buttons_update(TRUE, master, master,
+										gfte_is_older_notification(object),
+										gfte_is_younger_notification(object));
 			gfte_update_notification_page();
 			break;
+		}
 		case GFTE_PAGE_ICON:
-			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE);
+			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE,
+										gfte_is_older_item(object),
+										gfte_is_younger_item(object));
 			gfte_update_icon_page();
 			break;
 		case GFTE_PAGE_IMAGE:
-			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE);
+			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE,
+										gfte_is_older_item(object),
+										gfte_is_younger_item(object));
 			gfte_update_image_page();
 			break;
 		case GFTE_PAGE_TEXT:
-			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE);
+			gfte_toolbar_buttons_update(TRUE, TRUE, TRUE,
+										gfte_is_older_item(object),
+										gfte_is_younger_item(object));
 			gfte_update_text_page();
 			break;
 		case GFTE_PAGE_TOTAL:
@@ -2061,6 +2385,12 @@ gfte_make_notification_page() {
 	page = gtk_vbox_new(FALSE, 4);
 	gtk_container_set_border_width(GTK_CONTAINER(page), 12);
 
+	editor.notification.use_gtk =
+					gfte_add_check(page, GFTE_FLAGS_OBJECT,
+								   _("Use Gtk theme background"),
+								   gf_notification_get_use_gtk,
+								   gf_notification_set_use_gtk);
+
 	editor.notification.filename =
 					gfte_add_entry(page, sg, GFTE_FLAGS_OBJECT, _("Background:"),
 								   gf_notification_get_background,
@@ -2074,6 +2404,18 @@ gfte_make_notification_page() {
 									gf_notification_get_background,
 									gf_notification_set_background);
 
+	editor.notification.width =
+					gfte_add_spin_button(page, sg, GFTE_FLAGS_OBJECT, _("Width:"),
+										 GF_NOTIFICATION_MIN, GF_NOTIFICATION_MAX,
+										 gf_notification_get_width,
+										 gf_notification_set_width);
+
+	editor.notification.height =
+					gfte_add_spin_button(page, sg, GFTE_FLAGS_OBJECT, _("Height:"),
+										 GF_NOTIFICATION_MIN, GF_NOTIFICATION_MAX,
+										 gf_notification_get_height,
+										 gf_notification_set_width);
+
 	return page;
 }
 
@@ -2276,10 +2618,19 @@ gfte_build_toolbar() {
 
 	gfte_toolbar_separator_new(hbox);
 
+	editor.tool_up = 
+				gfte_toolbar_button_new(hbox, GTK_STOCK_GO_UP, _("Move up"),
+										G_CALLBACK(gfte_move_up), NULL);
+	editor.tool_down =
+				gfte_toolbar_button_new(hbox, GTK_STOCK_GO_DOWN, _("Move down"),
+										G_CALLBACK(gfte_move_down), NULL);
+
+	gfte_toolbar_separator_new(hbox);
+
 	help = gfte_toolbar_button_new(hbox, GTK_STOCK_HELP, _("Help"),
 								   G_CALLBACK(gfte_help), NULL);
 
-	gfte_toolbar_buttons_update(FALSE, FALSE, FALSE);
+	gfte_toolbar_buttons_update(FALSE, FALSE, FALSE, FALSE, FALSE);
 }
 
 static void
